"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrismaQueryRunner = void 0;
const AbstractQueryRunner_1 = require("./AbstractQueryRunner");
function isPrisma3(connection) {
    return connection.$executeRawUnsafe;
}
class PrismaQueryRunner extends AbstractQueryRunner_1.AbstractQueryRunner {
    constructor(connection, config) {
        super();
        this.transactionLevel = 0;
        this.config = config;
        this.connection = connection;
        switch (connection._activeProvider) {
            case 'postgresql':
                this.database = 'postgreSql';
                break;
            case 'mysql':
                this.database = 'mySql';
                break;
            case 'sqlite':
                this.database = 'sqlite';
                break;
            case 'sqlserver':
                this.database = 'sqlServer';
                break;
            default:
                throw new Error('Unknown Prisma provider of name ' + connection._activeProvider);
        }
        try {
            if (connection.$transaction) {
                this.transaction = undefined;
            }
        }
        catch {
            // This is running in an interactiveTransaction
            this.transaction = connection;
            this.transactionLevel = 1;
        }
        if (config?.forUseInTransaction) {
            this.transactionLevel = 1;
        }
    }
    useDatabase(database) {
        if (database !== this.database) {
            if (this.database === 'mySql' && database === 'mariaDB') {
                // @ts-ignore
                this.database = database;
            }
            else if (this.database === 'mariaDB' && database === 'mySql') {
                // @ts-ignore
                this.database = database;
            }
            else if (this.database === 'mySql' || this.database === 'mariaDB') {
                throw new Error('Unsupported database: ' + database + '. The current connection used in PrismaQueryRunner only supports mySql or mariaDB databases');
            }
            else {
                throw new Error('Unsupported database: ' + database + '. The current connection used in PrismaQueryRunner only supports ' + this.database + ' databases');
            }
        }
    }
    getNativeRunner() {
        return this.connection;
    }
    getCurrentNativeTransaction() {
        return this.transaction;
    }
    execute(fn) {
        return fn(this.connection, this.transaction);
    }
    executeQueryReturning(query, params) {
        const connection = this.transaction || this.connection;
        let result;
        if (isPrisma3(connection)) {
            result = connection.$queryRawUnsafe(query, ...params);
        }
        else {
            result = connection.$queryRaw(query, ...params);
        }
        return this.wrapPrismaPromise(result);
    }
    executeMutation(query, params) {
        const connection = this.transaction || this.connection;
        let result;
        if (isPrisma3(connection)) {
            result = connection.$executeRawUnsafe(query, ...params);
        }
        else {
            result = connection.$executeRaw(query, ...params);
        }
        return this.wrapPrismaPromise(result);
    }
    executeInsertReturningLastInsertedId(query, params = []) {
        if (this.database === 'mySql' || this.database === 'mariaDB') {
            if (this.containsInsertReturningClause(query, params)) {
                return super.executeInsertReturningLastInsertedId(query, params);
            }
            return this.executeCombined(() => this.executeInsert(query, params), () => this.executeSelectOneColumnOneRow('select last_insert_id()', [])).then(([_count, id]) => id);
        }
        if (this.database === 'sqlite') {
            if (this.containsInsertReturningClause(query, params)) {
                return super.executeInsertReturningLastInsertedId(query, params);
            }
            return this.executeCombined(() => this.executeInsert(query, params), () => this.executeSelectOneColumnOneRow('select last_insert_rowid()', [])).then(([_count, id]) => id);
        }
        return super.executeInsertReturningLastInsertedId(query, params);
    }
    executeBeginTransaction() {
        if (isPrisma3(this.connection)) {
            return Promise.reject(new Error('Low level transaction management is not supported by Prisma.'));
        }
        return Promise.reject(new Error('Long running transactions are not supported by Prisma. See https://github.com/prisma/prisma/issues/1844'));
    }
    executeCommit() {
        if (isPrisma3(this.connection)) {
            return Promise.reject(new Error('Low level transaction management is not supported by Prisma.'));
        }
        return Promise.reject(new Error('Long running transactions are not supported by Prisma. See https://github.com/prisma/prisma/issues/1844'));
    }
    executeRollback() {
        if (isPrisma3(this.connection)) {
            return Promise.reject(new Error('Low level transaction management is not supported by Prisma.'));
        }
        return Promise.reject(new Error('Long running transactions are not supported by Prisma. See https://github.com/prisma/prisma/issues/1844'));
    }
    isTransactionActive() {
        return this.transactionLevel > 0;
    }
    executeInTransaction(fn, _outermostQueryRunner) {
        if (this.transaction) {
            throw new Error('Nested interactive transaction is not supported by Prisma');
        }
        return this.connection.$transaction((interactiveTransactions) => {
            // @ts-ignore
            this.transaction = interactiveTransactions;
            this.transactionLevel++;
            const promises = fn();
            let result;
            if (Array.isArray(promises)) {
                result = Promise.all(promises);
            }
            else {
                result = promises;
            }
            return result.finally(() => {
                this.transactionLevel--;
                // @ts-ignore
                this.transaction = undefined;
            });
        }, this.config?.interactiveTransactionsOptions);
    }
    addParam(params, value) {
        const index = params.length;
        let result;
        switch (this.database) {
            case 'mariaDB':
                result = '?';
                break;
            case 'mySql':
                result = '?';
                break;
            case 'noopDB':
                result = '$' + index;
                break;
            case 'oracle':
                result = ':' + index;
                break;
            case 'postgreSql':
                result = '$' + (index + 1);
                break;
            case 'sqlite':
                result = '?';
                break;
            case 'sqlServer':
                result = '@P' + (index + 1);
                break;
            default:
                throw new Error('Unknown database ' + this.database);
        }
        params.push(value);
        return result;
    }
    createResolvedPromise(result) {
        return Promise.resolve(result);
    }
    executeCombined(fn1, fn2) {
        return fn1().then((r1) => {
            return fn2().then((r2) => {
                return [r1, r2];
            });
        });
    }
    lowLevelTransactionManagementSupported() {
        return false;
    }
    wrapPrismaPromise(promise) {
        // Use a real Promise instead of Prisma proxy to avoid issues due then with one param is not properly managed
        return new Promise((resolve, reject) => {
            promise.then(resolve, reject);
        });
    }
}
exports.PrismaQueryRunner = PrismaQueryRunner;
